@@ -158,6 +158,12 @@ class TestServersTagsV226(base.ClientTestBase):
         self.assertEqual(["t1", "t2", "t3"],
                          self.client.servers.tag_list(uuid))
 
+    def test_add_many(self):
+        uuid = self._boot_server_with_tags()
+        self.nova("server-tag-add %s t3 t4" % uuid)
+        self.assertEqual(["t1", "t2", "t3", "t4"],
+                         self.client.servers.tag_list(uuid))
+
     def test_set(self):
         uuid = self._boot_server_with_tags()
         self.nova("server-tag-set %s t3 t4" % uuid)
@@ -168,6 +174,11 @@ class TestServersTagsV226(base.ClientTestBase):
         self.nova("server-tag-delete %s t2" % uuid)
         self.assertEqual(["t1"], self.client.servers.tag_list(uuid))
 
+    def test_delete_many(self):
+        uuid = self._boot_server_with_tags()
+        self.nova("server-tag-delete %s t1 t2" % uuid)
+        self.assertEqual([], self.client.servers.tag_list(uuid))
+
     def test_delete_all(self):
         uuid = self._boot_server_with_tags()
         self.nova("server-tag-delete-all %s" % uuid)
@@ -194,6 +205,12 @@ class TestServersAutoAllocateNetworkCLI(base.ClientTestBase):
     def test_boot_server_with_auto_network(self):
         """Tests that the CLI defaults to 'auto' when --nic isn't specified.
         """
+        # check to see if multiple networks are available because if so we
+        # have to skip this test as auto will fail with a 409 conflict as it's
+        # an ambiguous request and nova won't know which network to pick
+        if self.multiple_networks:
+            # we could potentially get around this by extending TenantTestBase
+            self.skipTest('multiple networks available')
         server_info = self.nova('boot', params=(
             '%(name)s --flavor %(flavor)s --poll '
             '--image %(image)s ' % {'name': self.name_generate('server'),
