@@ -15,10 +15,10 @@
 # limitations under the License.
 
 import datetime
+import re
 
 import mock
 from oslo_utils import strutils
-import re
 import six
 from six.moves.urllib import parse
 
@@ -56,28 +56,30 @@ FAKE_RESPONSE_HEADERS = {'x-openstack-request-id': FAKE_REQUEST_ID}
 
 class FakeClient(fakes.FakeClient, client.Client):
 
-    def __init__(self, api_version=None, *args, **kwargs):
-        client.Client.__init__(self, 'username', 'password',
-                               'project_id', 'auth_url',
+    def __init__(self, api_version, *args, **kwargs):
+        client.Client.__init__(self, username='username', password='password',
+                               project_id='project_id', auth_url='auth_url',
                                extensions=kwargs.get('extensions'),
-                               direct_use=False)
-        self.api_version = api_version or api_versions.APIVersion("2.1")
-        kwargs["api_version"] = self.api_version
-        self.client = FakeHTTPClient(**kwargs)
+                               direct_use=False, api_version=api_version)
+        self.client = FakeSessionClient(api_version=api_version, **kwargs)
 
 
-class FakeHTTPClient(base_client.HTTPClient):
+class FakeSessionClient(base_client.SessionClient):
+
+    def __init__(self, *args, **kwargs):
 
-    def __init__(self, **kwargs):
-        self.username = 'username'
-        self.password = 'password'
-        self.auth_url = 'auth_url'
-        self.tenant_id = 'tenant_id'
         self.callstack = []
-        self.projectid = 'projectid'
-        self.user = 'user'
-        self.region_name = 'region_name'
-
+        self.visited = []
+        self.auth = mock.Mock()
+        self.session = mock.Mock()
+        self.service_type = 'service_type'
+        self.service_name = None
+        self.endpoint_override = None
+        self.interface = None
+        self.region_name = None
+        self.version = None
+        self.api_version = kwargs.get('api_version')
+        self.auth.get_auth_ref.return_value.project_id = 'tenant_id'
         # determines which endpoint to return in get_endpoint()
         # NOTE(augustina): this is a hacky workaround, ultimately
         # we need to fix our whole mocking architecture (fixtures?)
@@ -85,17 +87,19 @@ class FakeHTTPClient(base_client.HTTPClient):
             self.endpoint_type = kwargs['endpoint_type']
         else:
             self.endpoint_type = 'endpoint_type'
+        self.logger = mock.MagicMock()
 
-        self.service_type = 'service_type'
-        self.service_name = 'service_name'
-        self.volume_service_name = 'volume_service_name'
-        self.timings = 'timings'
-        self.bypass_url = 'bypass_url'
-        self.os_cache = 'os_cache'
-        self.http_log_debug = 'http_log_debug'
-        self.last_request_id = None
-        self.management_url = self.get_endpoint()
-        self.api_version = kwargs.get("api_version")
+    def get_endpoint(self, **kwargs):
+        # check if endpoint matches expected format (eg, v2.1)
+        if (hasattr(self, 'endpoint_type') and
+                ENDPOINT_TYPE_RE.search(self.endpoint_type)):
+            return "http://nova-api:8774/%s/" % self.endpoint_type
+        else:
+            return (
+                "http://nova-api:8774/v2.1/190a755eef2e4aac9f06aa6be9786385")
+
+    def request(self, url, method, **kwargs):
+        return self._cs_request(url, method, **kwargs)
 
     def _cs_request(self, url, method, **kwargs):
         # Check that certain things are called correctly
@@ -136,12 +140,21 @@ class FakeHTTPClient(base_client.HTTPClient):
             v2_image = True
             callback = callback.replace('get_v2_', 'get_')
 
+        simulate_pagination_next_links = [
+            'get_os_simple_tenant_usage',
+            'get_os_simple_tenant_usage_tenant_id',
+        ]
+        if callback in simulate_pagination_next_links:
+            while callback in self.visited:
+                callback += '_next'
+
         if not hasattr(self, callback):
             raise AssertionError('Called unknown API method: %s %s, '
                                  'expected fakes method name: %s' %
                                  (method, url, callback))
 
         # Note the call
+        self.visited.append(callback)
         self.callstack.append((method, url, kwargs.get('body')))
 
         status, headers, body = getattr(self, callback)(**kwargs)
@@ -158,15 +171,6 @@ class FakeHTTPClient(base_client.HTTPClient):
         })
         return r, body
 
-    def get_endpoint(self, **kwargs):
-        # check if endpoint matches expected format (eg, v2.1)
-        if (hasattr(self, 'endpoint_type') and
-                ENDPOINT_TYPE_RE.search(self.endpoint_type)):
-            return "http://nova-api:8774/%s/" % self.endpoint_type
-        else:
-            return (
-                "http://nova-api:8774/v2.1/190a755eef2e4aac9f06aa6be9786385")
-
     def get_versions(self):
         return (200, FAKE_RESPONSE_HEADERS, {
             "versions": [
@@ -380,14 +384,15 @@ class FakeHTTPClient(base_client.HTTPClient):
 
     def get_servers(self, **kw):
         return (200, {}, {"servers": [
-            {'id': 1234, 'name': 'sample-server'},
-            {'id': 5678, 'name': 'sample-server2'}
+            {'id': '1234', 'name': 'sample-server'},
+            {'id': '5678', 'name': 'sample-server2'},
+            {'id': '9014', 'name': 'help'}
         ]})
 
     def get_servers_detail(self, **kw):
         return (200, {}, {"servers": [
             {
-                "id": 1234,
+                "id": '1234',
                 "name": "sample-server",
                 "image": {
                     "id": FAKE_IMAGE_UUID_2,
@@ -428,7 +433,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                 "OS-EXT-MOD:some_thing": "mod_some_thing_value",
             },
             {
-                "id": 5678,
+                "id": '5678',
                 "name": "sample-server2",
                 "image": {
                     "id": FAKE_IMAGE_UUID_1,
@@ -472,7 +477,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                     }],
             },
             {
-                "id": 9012,
+                "id": '9012',
                 "name": "sample-server3",
                 "image": "",
                 "flavor": {
@@ -501,7 +506,7 @@ class FakeHTTPClient(base_client.HTTPClient):
                 }
             },
             {
-                "id": 9013,
+                "id": '9013',
                 "name": "sample-server4",
                 "flavor": {
                     "id": '80645cf4-6ad3-410a-bbc8-6f3e1e291f51',
@@ -512,6 +517,18 @@ class FakeHTTPClient(base_client.HTTPClient):
                 "hostId": "9e107d9d372bb6826bd81d3542a419d6",
                 "status": "ACTIVE",
             },
+            {
+                "id": '9014',
+                "name": "help",
+                "flavor": {
+                    "id": '80645cf4-6ad3-410a-bbc8-6f3e1e291f51',
+                },
+                "image": {
+                    "id": '3e861307-73a6-4d1f-8d68-f68b03223032',
+                },
+                "hostId": "9e107d9d372bb6826bd81d3542a419d6",
+                "status": "ACTIVE",
+            },
         ]})
 
     def post_servers(self, body, **kw):
@@ -552,7 +569,7 @@ class FakeHTTPClient(base_client.HTTPClient):
 
     def get_servers_1235(self, **kw):
         r = {'server': self.get_servers_detail()[2]['servers'][0]}
-        r['server']['id'] = 1235
+        r['server']['id'] = '1235'
         r['server']['status'] = 'error'
         r['server']['fault'] = {'message': 'something went wrong!'}
         return (200, {}, r)
@@ -569,10 +586,9 @@ class FakeHTTPClient(base_client.HTTPClient):
         r = {'server': self.get_servers_detail()[2]['servers'][3]}
         return (200, {}, r)
 
-    def put_servers_1234(self, body, **kw):
-        assert list(body) == ['server']
-        fakes.assert_has_keys(body['server'], optional=['name', 'adminPass'])
-        return (204, {}, body)
+    def get_servers_9014(self, **kw):
+        r = {'server': self.get_servers_detail()[2]['servers'][4]}
+        return (200, {}, r)
 
     def delete_os_server_groups_12345(self, **kw):
         return (202, {}, None)
@@ -586,9 +602,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_servers_5678(self, **kw):
         return (202, {}, None)
 
-    def delete_servers_1234_metadata_test_key(self, **kw):
-        return (204, {}, None)
-
     def delete_servers_1234_metadata_key1(self, **kw):
         return (204, {}, None)
 
@@ -598,9 +611,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     def post_servers_1234_metadata(self, **kw):
         return (204, {}, {'metadata': {'test_key': 'test_value'}})
 
-    def put_servers_1234_metadata_test_key(self, **kw):
-        return (200, {}, {'meta': {'test_key': 'test_value'}})
-
     def get_servers_1234_diagnostics(self, **kw):
         return (200, {}, {'data': 'Fake diagnostics'})
 
@@ -1093,18 +1103,12 @@ class FakeHTTPClient(base_client.HTTPClient):
     #
     # Images
     #
-    def get_images(self, **kw):
-        return (200, {}, {'images': [
-            {'id': FAKE_IMAGE_UUID_1, 'name': 'CentOS 5.2'},
-            {'id': FAKE_IMAGE_UUID_2, 'name': 'My Server Backup'}
-        ]})
-
     def get_images_detail(self, **kw):
         return (200, {}, {'images': [
             {
                 "id": FAKE_IMAGE_UUID_SNAPSHOT,
                 "name": "My Server Backup",
-                "serverId": 1234,
+                "serverId": '1234',
                 "updated": "2010-10-10T12:00:00Z",
                 "created": "2010-08-10T12:00:00Z",
                 "status": "SAVING",
@@ -1114,7 +1118,7 @@ class FakeHTTPClient(base_client.HTTPClient):
             {
                 "id": FAKE_IMAGE_UUID_SNAP_DEL,
                 "name": "My Server Backup Deleted",
-                "serverId": 1234,
+                "serverId": '1234',
                 "updated": "2010-10-10T12:00:00Z",
                 "created": "2010-08-10T12:00:00Z",
                 "status": "DELETED",
@@ -1135,7 +1139,7 @@ class FakeHTTPClient(base_client.HTTPClient):
             {
                 "id": FAKE_IMAGE_UUID_2,
                 "name": "My Server Backup",
-                "serverId": 1234,
+                "serverId": '1234',
                 "updated": "2010-10-10T12:00:00Z",
                 "created": "2010-08-10T12:00:00Z",
                 "status": "SAVING",
@@ -1537,15 +1541,6 @@ class FakeHTTPClient(base_client.HTTPClient):
              self.get_os_security_group_rules()[2]['security_group_rules'][0]}
         return (202, {}, r)
 
-    #
-    # Security Group Default Rules
-    #
-    def get_os_security_group_default_rules(self, **kw):
-        return (200, {}, {"security_group_default_rules": [
-            {'id': 1, 'ip_protocol': 'TCP', 'from_port': 22,
-             'to_port': 22, 'cidr': '10.0.0.0/8'}
-        ]})
-
     #
     # Tenant Usage
     #
@@ -1566,6 +1561,8 @@ class FakeHTTPClient(base_client.HTTPClient):
                         six.u('name'): six.u('f15image1'),
                         six.u('tenant_id'):
                             six.u('7b0a1d73f8fb41718f3343c207597869'),
+                        six.u('instance_id'):
+                            six.u('f079e394-1111-457b-b350-bb5ecc685cdd'),
                         six.u('vcpus'): 1,
                         six.u('memory_mb'): 512,
                         six.u('state'): six.u('active'),
@@ -1575,6 +1572,37 @@ class FakeHTTPClient(base_client.HTTPClient):
                     six.u('start'): six.u('2011-12-25 19:48:41.750687'),
                     six.u('total_local_gb_usage'): 0.0}]})
 
+    def get_os_simple_tenant_usage_next(self, **kw):
+        return (200, FAKE_RESPONSE_HEADERS,
+                {six.u('tenant_usages'): [{
+                    six.u('total_memory_mb_usage'): 25451.762807466665,
+                    six.u('total_vcpus_usage'): 49.71047423333333,
+                    six.u('total_hours'): 49.71047423333333,
+                    six.u('tenant_id'):
+                        six.u('7b0a1d73f8fb41718f3343c207597869'),
+                    six.u('stop'): six.u('2012-01-22 19:48:41.750722'),
+                    six.u('server_usages'): [{
+                        six.u('hours'): 49.71047423333333,
+                        six.u('uptime'): 27035,
+                        six.u('local_gb'): 0,
+                        six.u('ended_at'): None,
+                        six.u('name'): six.u('f15image1'),
+                        six.u('tenant_id'):
+                            six.u('7b0a1d73f8fb41718f3343c207597869'),
+                        six.u('instance_id'):
+                            six.u('f079e394-2222-457b-b350-bb5ecc685cdd'),
+                        six.u('vcpus'): 1,
+                        six.u('memory_mb'): 512,
+                        six.u('state'): six.u('active'),
+                        six.u('flavor'): six.u('m1.tiny'),
+                        six.u('started_at'):
+                            six.u('2012-01-20 18:06:06.479998')}],
+                    six.u('start'): six.u('2011-12-25 19:48:41.750687'),
+                    six.u('total_local_gb_usage'): 0.0}]})
+
+    def get_os_simple_tenant_usage_next_next(self, **kw):
+        return (200, FAKE_RESPONSE_HEADERS, {six.u('tenant_usages'): []})
+
     def get_os_simple_tenant_usage_tenantfoo(self, **kw):
         return (200, FAKE_RESPONSE_HEADERS,
                 {six.u('tenant_usage'): {
@@ -1591,6 +1619,8 @@ class FakeHTTPClient(base_client.HTTPClient):
                         six.u('name'): six.u('f15image1'),
                         six.u('tenant_id'):
                             six.u('7b0a1d73f8fb41718f3343c207597869'),
+                        six.u('instance_id'):
+                            six.u('f079e394-1111-457b-b350-bb5ecc685cdd'),
                         six.u('vcpus'): 1, six.u('memory_mb'): 512,
                         six.u('state'): six.u('active'),
                         six.u('flavor'): six.u('m1.tiny'),
@@ -1612,6 +1642,8 @@ class FakeHTTPClient(base_client.HTTPClient):
                 six.u('ended_at'): None,
                 six.u('name'): six.u('f15image1'),
                 six.u('tenant_id'): six.u('7b0a1d73f8fb41718f3343c207597869'),
+                six.u('instance_id'):
+                    six.u('f079e394-1111-457b-b350-bb5ecc685cdd'),
                 six.u('vcpus'): 1, six.u('memory_mb'): 512,
                 six.u('state'): six.u('active'),
                 six.u('flavor'): six.u('m1.tiny'),
@@ -1632,6 +1664,8 @@ class FakeHTTPClient(base_client.HTTPClient):
                 six.u('ended_at'): None,
                 six.u('name'): six.u('f15image1'),
                 six.u('tenant_id'): six.u('7b0a1d73f8fb41718f3343c207597869'),
+                six.u('instance_id'):
+                    six.u('f079e394-1111-457b-b350-bb5ecc685cdd'),
                 six.u('vcpus'): 1, six.u('memory_mb'): 512,
                 six.u('state'): six.u('active'),
                 six.u('flavor'): six.u('m1.tiny'),
@@ -1639,12 +1673,37 @@ class FakeHTTPClient(base_client.HTTPClient):
             six.u('start'): six.u('2011-12-25 19:48:41.750687'),
             six.u('total_local_gb_usage'): 0.0}})
 
+    def get_os_simple_tenant_usage_tenant_id_next(self, **kw):
+        return (200, {}, {six.u('tenant_usage'): {
+            six.u('total_memory_mb_usage'): 25451.762807466665,
+            six.u('total_vcpus_usage'): 49.71047423333333,
+            six.u('total_hours'): 49.71047423333333,
+            six.u('tenant_id'): six.u('7b0a1d73f8fb41718f3343c207597869'),
+            six.u('stop'): six.u('2012-01-22 19:48:41.750722'),
+            six.u('server_usages'): [{
+                six.u('hours'): 49.71047423333333,
+                six.u('uptime'): 27035, six.u('local_gb'): 0,
+                six.u('ended_at'): None,
+                six.u('name'): six.u('f15image1'),
+                six.u('tenant_id'): six.u('7b0a1d73f8fb41718f3343c207597869'),
+                six.u('instance_id'):
+                    six.u('f079e394-2222-457b-b350-bb5ecc685cdd'),
+                six.u('vcpus'): 1, six.u('memory_mb'): 512,
+                six.u('state'): six.u('active'),
+                six.u('flavor'): six.u('m1.tiny'),
+                six.u('started_at'): six.u('2012-01-20 18:06:06.479998')}],
+            six.u('start'): six.u('2011-12-25 19:48:41.750687'),
+            six.u('total_local_gb_usage'): 0.0}})
+
+    def get_os_simple_tenant_usage_tenant_id_next_next(self, **kw):
+        return (200, {}, {six.u('tenant_usage'): {}})
+
     #
     # Aggregates
     #
 
     def get_os_aggregates(self, *kw):
-        return (200, {}, {"aggregates": [
+        response = (200, {}, {"aggregates": [
             {'id': '1',
              'name': 'test',
              'availability_zone': 'nova1'},
@@ -1655,6 +1714,13 @@ class FakeHTTPClient(base_client.HTTPClient):
              'name': 'test3',
              'metadata': {'test': "dup", "none_key": "Nine"}},
         ]})
+        # microversion >= 2.41 returns the uuid in the response
+        if self.api_version >= api_versions.APIVersion('2.41'):
+            aggregates = response[2]['aggregates']
+            aggregates[0]['uuid'] = '80785864-087b-45a5-a433-b20eac9b58aa'
+            aggregates[1]['uuid'] = '30827713-5957-4b68-8fd3-ccaddb568c24'
+            aggregates[2]['uuid'] = '9a651b22-ce3f-4a87-acd7-98446ef591c4'
+        return response
 
     def _return_aggregate(self):
         r = {'aggregate': self.get_os_aggregates()[2]['aggregates'][0]}
@@ -2045,95 +2111,6 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_servers_1234_os_interface_port_id(self, **kw):
         return (200, {}, None)
 
-    # NOTE (vkhomenko):
-    # Volume responses was taken from:
-    # https://wiki.openstack.org/wiki/CreateVolumeFromImage
-    # http://jorgew.github.com/block-storage-api/content/
-    # GET_listDetailVolumes_v1__tenantId__volumes_detail_.html
-    # I suppose they are outdated and should be updated after Cinder released
-
-    def get_volumes_detail(self, **kw):
-        return (200, FAKE_RESPONSE_HEADERS, {"volumes": [
-            {
-                "display_name": "Work",
-                "display_description": "volume for work",
-                "status": "ATTACHED",
-                "id": "15e59938-07d5-11e1-90e3-e3dffe0c5983",
-                "created_at": "2011-09-09T00:00:00Z",
-                "attached": "2011-11-11T00:00:00Z",
-                "size": 1024,
-                "attachments": [
-                    {"id": "3333",
-                     "links": ''}],
-                "metadata": {}},
-            {
-                "display_name": "Work2",
-                "display_description": "volume for work2",
-                "status": "ATTACHED",
-                "id": "15e59938-07d5-11e1-90e3-ee32ba30feaa",
-                "created_at": "2011-09-09T00:00:00Z",
-                "attached": "2011-11-11T00:00:00Z",
-                "size": 1024,
-                "attachments": [
-                    {"id": "2222",
-                     "links": ''}],
-                "metadata": {}}]})
-
-    def get_volumes(self, **kw):
-        return (200, FAKE_RESPONSE_HEADERS, {"volumes": [
-            {
-                "display_name": "Work",
-                "display_description": "volume for work",
-                "status": "ATTACHED",
-                "id": "15e59938-07d5-11e1-90e3-e3dffe0c5983",
-                "created_at": "2011-09-09T00:00:00Z",
-                "attached": "2011-11-11T00:00:00Z",
-                "size": 1024,
-                "attachments": [
-                    {"id": "3333",
-                     "links": ''}],
-                "metadata": {}},
-            {
-                "display_name": "Work2",
-                "display_description": "volume for work2",
-                "status": "ATTACHED",
-                "id": "15e59938-07d5-11e1-90e3-ee32ba30feaa",
-                "created_at": "2011-09-09T00:00:00Z",
-                "attached": "2011-11-11T00:00:00Z",
-                "size": 1024,
-                "attachments": [
-                    {"id": "2222",
-                     "links": ''}],
-                "metadata": {}}]})
-
-    def get_volumes_15e59938_07d5_11e1_90e3_e3dffe0c5983(self, **kw):
-        return (200, FAKE_RESPONSE_HEADERS, {
-                "volume": self.get_volumes_detail()[2]['volumes'][0]})
-
-    def get_volumes_15e59938_07d5_11e1_90e3_ee32ba30feaa(self, **kw):
-        return (200, {}, {
-                "volume": self.get_volumes_detail()[2]['volumes'][1]})
-
-    def post_volumes(self, **kw):
-        return (200, FAKE_RESPONSE_HEADERS, {"volume":
-                {"status": "creating",
-                 "display_name": "vol-007",
-                 "attachments": [(0)],
-                 "availability_zone": "cinder",
-                 "created_at": "2012-08-13T10:57:17.000000",
-                 "display_description": "create volume from image",
-                 "image_id": "f4cf905f-7c58-4d7b-8314-8dd8a2d1d483",
-                 "volume_type": "None",
-                 "metadata": {},
-                 "id": "5cb239f6-1baf-4fe1-bd78-c852cf00fa39",
-                 "size": 1}})
-
-    def delete_volumes_15e59938_07d5_11e1_90e3_e3dffe0c5983(self, **kw):
-        return (200, FAKE_RESPONSE_HEADERS, {})
-
-    def delete_volumes_15e59938_07d5_11e1_90e3_ee32ba30feaa(self, **kw):
-        return (200, {}, {})
-
     def post_servers_1234_os_volume_attachments(self, **kw):
         return (200, FAKE_RESPONSE_HEADERS, {
             "volumeAttachment":
@@ -2241,7 +2218,7 @@ class FakeHTTPClient(base_client.HTTPClient):
             "dest_compute": "compute2",
             "dest_host": "1.2.3.4",
             "dest_node": "node2",
-            "id": 1234,
+            "id": '1234',
             "instance_uuid": "instance_id_123",
             "new_instance_type_id": 2,
             "old_instance_type_id": 1,
@@ -2258,31 +2235,47 @@ class FakeHTTPClient(base_client.HTTPClient):
 
         return (200, FAKE_RESPONSE_HEADERS, migrations)
 
-    def post_os_server_external_events(self, **kw):
-        return (200, {}, {'events': [
-            {'name': 'network-changed',
-             'server_uuid': '1234'}]})
-
     #
     # Server Groups
     #
 
-    def get_os_server_groups(self, *kw):
-        return (200, {},
-                {"server_groups": [
-                 {"members": [], "metadata": {},
-                  "id": "2cbd51f4-fafe-4cdb-801b-cf913a6f288b",
-                  "policies": [], "name": "ig1"},
-                 {"members": [], "metadata": {},
-                  "id": "4473bb03-4370-4bfb-80d3-dc8cffc47d94",
-                  "policies": ["anti-affinity"], "name": "ig2"},
-                 {"members": [], "metadata": {"key": "value"},
-                  "id": "31ab9bdb-55e1-4ac3-b094-97eeb1b65cc4",
-                  "policies": [], "name": "ig3"},
-                 {"members": ["2dccb4a1-02b9-482a-aa23-5799490d6f5d"],
-                  "metadata": {},
-                  "id": "4890bb03-7070-45fb-8453-d34556c87d94",
-                  "policies": ["anti-affinity"], "name": "ig2"}]})
+    def get_os_server_groups(self, **kw):
+        server_groups = [
+            {"members": [], "metadata": {},
+             "id": "2cbd51f4-fafe-4cdb-801b-cf913a6f288b",
+             "policies": [], "name": "ig1"},
+            {"members": [], "metadata": {},
+             "id": "4473bb03-4370-4bfb-80d3-dc8cffc47d94",
+             "policies": ["anti-affinity"], "name": "ig2"},
+            {"members": [], "metadata": {"key": "value"},
+             "id": "31ab9bdb-55e1-4ac3-b094-97eeb1b65cc4",
+             "policies": [], "name": "ig3"},
+            {"members": ["2dccb4a1-02b9-482a-aa23-5799490d6f5d"],
+             "metadata": {},
+             "id": "4890bb03-7070-45fb-8453-d34556c87d94",
+             "policies": ["anti-affinity"], "name": "ig2"}]
+
+        other_project_server_groups = [
+            {"members": [], "metadata": {},
+             "id": "11111111-1111-1111-1111-111111111111",
+             "policies": [], "name": "ig4"},
+            {"members": [], "metadata": {},
+             "id": "22222222-2222-2222-2222-222222222222",
+             "policies": ["anti-affinity"], "name": "ig5"},
+            {"members": [], "metadata": {"key": "value"},
+             "id": "31ab9bdb-55e1-4ac3-b094-97eeb1b65cc4",
+             "policies": [], "name": "ig6"},
+            {"members": ["33333333-3333-3333-3333-333333333333"],
+             "metadata": {},
+             "id": "44444444-4444-4444-4444-444444444444",
+             "policies": ["anti-affinity"], "name": "ig5"}]
+
+        if kw.get("all_projects", False):
+            server_groups.extend(other_project_server_groups)
+        limit = int(kw.get("limit", 1000))
+        offset = int(kw.get("offset", 0))
+        server_groups = server_groups[offset:limit + 1]
+        return (200, {}, {"server_groups": server_groups})
 
     def _return_server_group(self):
         r = {'server_group':
@@ -2346,6 +2339,15 @@ class FakeHTTPClient(base_client.HTTPClient):
     def put_servers_1234_tags_tag(self, **kw):
         return (201, {}, None)
 
+    def put_servers_1234_tags_tag1(self, **kw):
+        return (201, {}, None)
+
+    def put_servers_1234_tags_tag2(self, **kw):
+        return (201, {}, None)
+
+    def put_servers_1234_tags_tag3(self, **kw):
+        return (201, {}, None)
+
     def put_servers_1234_tags(self, **kw):
         return (201, {}, None)
 
@@ -2355,41 +2357,98 @@ class FakeHTTPClient(base_client.HTTPClient):
     def delete_servers_1234_tags_tag(self, **kw):
         return (204, {}, None)
 
+    def delete_servers_1234_tags_tag1(self, **kw):
+        return (204, {}, None)
+
+    def delete_servers_1234_tags_tag2(self, **kw):
+        return (204, {}, None)
+
+    def delete_servers_1234_tags_tag3(self, **kw):
+        return (204, {}, None)
+
     def delete_servers_1234_tags(self, **kw):
         return (204, {}, None)
 
-    def get_servers_1234_tags_tag(self, **kw):
-        return (204, {}, None)
+    def get_os_tenant_networks(self):
+        return (200, FAKE_RESPONSE_HEADERS, {
+            'networks': [{"label": "1", "cidr": "10.0.0.0/24",
+                          'project_id': '4ffc664c198e435e9853f2538fbcd7a7',
+                          'id': '1'}]})
 
+    def get_os_tenant_networks_1(self, **kw):
+        return (200, FAKE_RESPONSE_HEADERS, {
+            'network': {"label": "1", "cidr": "10.0.0.0/24",
+                        'project_id': '4ffc664c198e435e9853f2538fbcd7a7',
+                        'id': '1'}})
 
-class FakeSessionClient(fakes.FakeClient, client.Client):
+    def post_os_tenant_networks(self, **kw):
+        return (201, FAKE_RESPONSE_HEADERS, {
+            'network': {"label": "1", "cidr": "10.0.0.0/24",
+                        'project_id': '4ffc664c198e435e9853f2538fbcd7a7',
+                        'id': '1'}})
 
-    def __init__(self, api_version, *args, **kwargs):
-        client.Client.__init__(self, 'username', 'password',
-                               'project_id', 'auth_url',
-                               extensions=kwargs.get('extensions'),
-                               api_version=api_version, direct_use=False)
-        kwargs['api_version'] = api_version
-        self.client = FakeSessionMockClient(**kwargs)
+    def delete_os_tenant_networks_1(self, **kw):
+        return (204, FAKE_RESPONSE_HEADERS, None)
 
+    def get_os_baremetal_nodes(self, **kw):
+        return (
+            200, FAKE_RESPONSE_HEADERS, {
+                'nodes': [
+                    {
+                        "id": 1,
+                        "instance_uuid": None,
+                        "interfaces": [],
+                        "cpus": 2,
+                        "local_gb": 10,
+                        "memory_mb": 5,
+                        "pm_address": "2.3.4.5",
+                        "pm_user": "pmuser",
+                        "pm_password": "pmpass",
+                        "prov_mac_address": "aa:bb:cc:dd:ee:ff",
+                        "prov_vlan_id": 1,
+                        "service_host": "somehost",
+                        "terminal_port": 8080,
+                    }
+                ]
+            }
+        )
 
-class FakeSessionMockClient(base_client.SessionClient, FakeHTTPClient):
+    def get_os_baremetal_nodes_1(self, **kw):
+        return (
+            200, FAKE_RESPONSE_HEADERS, {
+                'node': {
+                    "id": 1,
+                    "instance_uuid": None,
+                    "pm_address": "1.2.3.4",
+                    "interfaces": [],
+                    "cpus": 2,
+                    "local_gb": 10,
+                    "memory_mb": 5,
+                    "pm_user": "pmuser",
+                    "pm_password": "pmpass",
+                    "prov_mac_address": "aa:bb:cc:dd:ee:ff",
+                    "prov_vlan_id": 1,
+                    "service_host": "somehost",
+                    "terminal_port": 8080,
+                }
+            }
+        )
 
-    def __init__(self, *args, **kwargs):
+    def post_os_assisted_volume_snapshots(self, **kw):
+        return (202, FAKE_RESPONSE_HEADERS,
+                {'snapshot': {'id': 'blah', 'volumeId': '1'}})
 
-        self.callstack = []
-        self.auth = mock.Mock()
-        self.session = mock.Mock()
-        self.session.get_endpoint.return_value = FakeHTTPClient.get_endpoint(
-            self)
-        self.service_type = 'service_type'
-        self.service_name = None
-        self.endpoint_override = None
-        self.interface = None
-        self.region_name = None
-        self.version = None
-        self.api_version = kwargs.get('api_version')
-        self.auth.get_auth_ref.return_value.project_id = 'tenant_id'
+    def delete_os_assisted_volume_snapshots_x(self, **kw):
+        return (202, FAKE_RESPONSE_HEADERS, {})
 
-    def request(self, url, method, **kwargs):
-        return self._cs_request(url, method, **kwargs)
+    def post_os_server_external_events(self, **kw):
+        return (200, FAKE_RESPONSE_HEADERS, {
+            'events': [
+                {'name': 'test-event',
+                 'status': 'completed',
+                 'tag': 'tag',
+                 'server_uuid': 'fake-uuid1'},
+                {'name': 'test-event',
+                 'status': 'completed',
+                 'tag': 'tag',
+                 'server_uuid': 'fake-uuid2'}]})
